
#include <Arduino.h>
#include <Wire.h>
#include "Keyboard.h"
#include "Rfid.h"
#include "Interface.h"
#include <ESP8266WiFi.h>
#include <vector>
//#include "Users.h"
#include "UserDb.h"
#include "Persistence.h"
// File persistence
Persistence persistence;
userdb::UserDb userdatabase;
// Rfid
Rfid rfid;
// i2C Bus
#define PIN_WIRE_SDA D3
#define PIN_WIRE_SCL D4

Keyboard keyboard(200);

Interface iface;
// Wifi control
IPAddress local_IP(192, 168, 4, 22);
IPAddress gateway(192, 168, 4, 9);
IPAddress subnet(255, 255, 255, 0);

// User DB
// Users users(persistence);

void setup()
{
	Persistence::Configuration config = persistence.loadConfig();
	// users.ImportFromPersistence();
	Serial.begin(9600);
	Serial.println("Starting System");
	Serial.println("\t1. Network config: " + WiFi.softAPConfig(local_IP, gateway, subnet) ? "Ready" : "Failed!");
	if (strlen(config.PASS) > 0)
		Serial.println("\t1. AP setup with password: " + WiFi.softAP(config.SSID, config.PASS) ? "Ready" : "Failed!");
	else
		Serial.println("\t1. AP setup without password: " +WiFi.softAP(config.SSID) ? "Ready" : "Failed!");
	delay(250);
#ifdef DEBUG
	userdatabase.print_to_serial();
#endif
	keyboard.begin(&Wire);
	rfid.begin();
	iface.begin(&keyboard);
	userdatabase.remove_user_by_line("userdb.csv",2);
}

void loop()
{
	keyboard.scanAsync();
	rfid.scan();
	userdb::User login_user;
	if (iface.pinAvailable())
	{
		unsigned long delta = millis();
		login_user = userdatabase.user_by_pin(iface.getPin());
		Serial.println("Query duration: " +String(millis()-delta));
		if (login_user.enabled == false)
		{
			iface.showMessage("Login failed!", "-> Pin incorrect", 3000);
			return;
		}

	}
	else if (rfid.available())
	{
		unsigned long delta = millis();
		login_user = userdatabase.user_by_rfid(rfid.getID());
		Serial.println("Query duration: " +String(millis()-delta));
		if (login_user.enabled == false)
		{
			iface.showMessage("Login failed!", "-> Unkown Card", 3000);
			return;
		}

	}

	if (login_user.enabled == true)
	{
		iface.greetUser(login_user.first_name + " " + login_user.last_name);
		Serial.println("Logon from User " + login_user.toString());
	}
	iface.render();
}